@@ -18,15 +18,19 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "../lib/libcompat.h"
+
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
+
 #include "check_error.h"
 
 
-void eprintf (char *fmt, char *file, int line, ...)
+/* FIXME: including a colon at the end is a bad way to indicate an error */
+void eprintf (const char *fmt, const char *file, int line, ...)
 {
   va_list args;
   fflush(stderr);
@@ -49,7 +53,7 @@ void *emalloc (size_t n)
   void *p;
   p = malloc(n);
   if (p == NULL)
-    eprintf("malloc of %u bytes failed:", __FILE__, __LINE__, n);
+    eprintf("malloc of %u bytes failed:", __FILE__, __LINE__ - 2, n);
   return p;
 }
 
@@ -58,6 +62,6 @@ void *erealloc (void * ptr, size_t n)
   void *p;
   p = realloc (ptr, n);
   if (p == NULL)
-    eprintf("realloc of %u bytes failed:", __FILE__, __LINE__, n);
+    eprintf("realloc of %u bytes failed:", __FILE__, __LINE__ - 2, n);
   return p;
 }
