@@ -18,19 +18,21 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
+#include "../lib/libcompat.h"
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <sys/time.h>
-#include <time.h>
 #include <check.h>
+#if HAVE_SUBUNIT_CHILD_H
+#include <subunit/child.h>
+#endif
 
 #include "check_error.h"
 #include "check_list.h"
 #include "check_impl.h"
 #include "check_log.h"
 #include "check_print.h"
+#include "check_str.h"
 
 
 static void srunner_send_evt (SRunner *sr, void *obj, enum cl_event evt);
@@ -107,6 +109,13 @@ void log_suite_end (SRunner *sr, Suite *s)
   srunner_send_evt (sr, s, CLEND_S);
 }
 
+void log_test_start (SRunner *sr, TCase * tc, TF * tfun)
+{
+  char buffer[100];
+  snprintf(buffer, 99, "%s:%s", tc->name, tfun->name);
+  srunner_send_evt (sr, buffer, CLSTART_T);
+}
+
 void log_test_end (SRunner *sr, TestResult *tr)
 {
   srunner_send_evt (sr, tr, CLEND_T);
@@ -128,7 +137,6 @@ static void srunner_send_evt (SRunner *sr, void *obj, enum cl_event evt)
 void stdout_lfun (SRunner *sr, FILE *file, enum print_output printmode,
 		  void *obj, enum cl_event evt)
 {
-  TestResult *tr;
   Suite *s;
   
   if (printmode == CK_ENV) {
@@ -163,8 +171,9 @@ void stdout_lfun (SRunner *sr, FILE *file, enum print_output printmode,
   case CLEND_S:
     s = obj;
     break;
+  case CLSTART_T:
+    break;
   case CLEND_T:
-    tr = obj;
     break;
   default:
     eprintf("Bad event type received in stdout_lfun", __FILE__, __LINE__);
@@ -173,7 +182,7 @@ void stdout_lfun (SRunner *sr, FILE *file, enum print_output printmode,
   
 }
 
-void lfile_lfun (SRunner *sr, FILE *file, enum print_output printmode,
+void lfile_lfun (SRunner *sr, FILE *file, enum print_output printmode CK_ATTRIBUTE_UNUSED,
 		 void *obj, enum cl_event evt)
 {
   TestResult *tr;
@@ -197,18 +206,20 @@ void lfile_lfun (SRunner *sr, FILE *file, enum print_output printmode,
   case CLEND_S:
     s = obj;
     break;
+  case CLSTART_T:
+    break;
   case CLEND_T:
     tr = obj;
     tr_fprint(file, tr, CK_VERBOSE);
     break;
   default:
-    eprintf("Bad event type received in stdout_lfun", __FILE__, __LINE__);
+    eprintf("Bad event type received in lfile_lfun", __FILE__, __LINE__);
   }
 
   
 }
 
-void xml_lfun (SRunner *sr, FILE *file, enum print_output printmode,
+void xml_lfun (SRunner *sr CK_ATTRIBUTE_UNUSED, FILE *file, enum print_output printmode CK_ATTRIBUTE_UNUSED,
 		  void *obj, enum cl_event evt)
 {
   TestResult *tr;
@@ -250,6 +261,8 @@ void xml_lfun (SRunner *sr, FILE *file, enum print_output printmode,
     fprintf(file, "  </suite>\n");
     s = obj;
     break;
+  case CLSTART_T:
+    break;
   case CLEND_T:
     tr = obj;
     tr_xmlprint(file, tr, CK_VERBOSE);
@@ -260,6 +273,66 @@ void xml_lfun (SRunner *sr, FILE *file, enum print_output printmode,
 
 }
 
+#if ENABLE_SUBUNIT
+void subunit_lfun (SRunner *sr, FILE *file, enum print_output printmode,
+		  void *obj, enum cl_event evt)
+{
+  TestResult *tr;
+  Suite *s;
+  char const * name;
+  
+  /* assert(printmode == CK_SUBUNIT); */
+
+  switch (evt) {
+  case CLINITLOG_SR:
+    break;
+  case CLENDLOG_SR:
+    break;
+  case CLSTART_SR:
+    break;
+  case CLSTART_S:
+    s = obj;
+    break;
+  case CLEND_SR:
+    if (printmode > CK_SILENT) {
+      fprintf (file, "\n");
+      srunner_fprint (file, sr, printmode);
+    }
+    break;
+  case CLEND_S:
+    s = obj;
+    break;
+  case CLSTART_T:
+    name = obj;
+    subunit_test_start(name);
+    break;
+  case CLEND_T:
+    tr = obj;
+    {
+      char *name = ck_strdup_printf ("%s:%s", tr->tcname, tr->tname);
+      char *msg = tr_short_str (tr);
+      switch (tr->rtype) {
+	case CK_PASS:
+	  subunit_test_pass(name);
+	  break;
+	case CK_FAILURE:
+	  subunit_test_fail(name, msg);
+	  break;
+	case CK_ERROR:
+	  subunit_test_error(name, msg);
+	  break;
+	default:
+	  eprintf("Bad result type in subunit_lfun", __FILE__, __LINE__);
+        free(name);
+        free(msg);
+      }
+    }
+    break;
+  default:
+    eprintf("Bad event type received in subunit_lfun", __FILE__, __LINE__);
+  }
+}
+#endif
 
 FILE *srunner_open_lfile (SRunner *sr)
 {
@@ -267,7 +340,7 @@ FILE *srunner_open_lfile (SRunner *sr)
   if (srunner_has_log (sr)) {
     f = fopen(sr->log_fname, "w");
     if (f == NULL)
-      eprintf ("Could not open log file %s:", __FILE__, __LINE__,
+      eprintf ("Error in call to fopen while opening log file %s:", __FILE__, __LINE__ - 2,
 	       sr->log_fname);
   }
   return f;
@@ -279,7 +352,7 @@ FILE *srunner_open_xmlfile (SRunner *sr)
   if (srunner_has_xml (sr)) {
     f = fopen(sr->xml_fname, "w");
     if (f == NULL)
-      eprintf ("Could not open xml file %s:", __FILE__, __LINE__,
+      eprintf ("Error in call to fopen while opening xml file %s:", __FILE__, __LINE__ - 2,
 	       sr->xml_fname);
   }
   return f;
@@ -289,7 +362,14 @@ void srunner_init_logging (SRunner *sr, enum print_output print_mode)
 {
   FILE *f;
   sr->loglst = check_list_create();
-  srunner_register_lfun (sr, stdout, 0, stdout_lfun, print_mode);
+#if ENABLE_SUBUNIT
+  if (print_mode != CK_SUBUNIT)
+#endif
+      srunner_register_lfun (sr, stdout, 0, stdout_lfun, print_mode);
+#if ENABLE_SUBUNIT
+  else
+      srunner_register_lfun (sr, stdout, 0, subunit_lfun, print_mode);
+#endif
   f = srunner_open_lfile (sr);
   if (f) {
     srunner_register_lfun (sr, f, 1, lfile_lfun, print_mode);
@@ -314,7 +394,7 @@ void srunner_end_logging (SRunner *sr)
     if (lg->close) {
       rval = fclose (lg->lfile);
       if (rval != 0)
-	eprintf ("Error closing log file:", __FILE__, __LINE__);
+	eprintf ("Error in call to fclose while closing log file:", __FILE__, __LINE__ - 2);
     }
     free (lg);
   }
