@@ -18,16 +18,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
+#include "../lib/libcompat.h"
 
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
@@ -38,6 +34,13 @@
 #include "check_impl.h"
 #include "check_pack.h"
 
+#ifdef HAVE_PTHREAD
+pthread_mutex_t lock_mutex = PTHREAD_MUTEX_INITIALIZER;
+#else
+#define pthread_mutex_lock(arg)
+#define pthread_mutex_unlock(arg)
+#endif
+
 /* typedef an unsigned int that has at least 4 bytes */
 typedef uint32_t ck_uint32;
 
@@ -247,9 +250,13 @@ static void upack_fail (char **buf, FailMsg *fmsg)
 static void check_type (int type, const char *file, int line)
 {
   if (type < 0 || type >= CK_MSG_LAST)
-    eprintf ("Bad message type arg", file, line);
+    eprintf ("Bad message type arg %d", file, line, type);
 }
 
+#ifdef HAVE_PTHREAD
+pthread_mutex_t mutex_lock = PTHREAD_MUTEX_INITIALIZER;
+#endif
+
 void ppack (int fdes, enum ck_msg_type type, CheckMsg *msg)
 {
   char *buf;
@@ -257,7 +264,9 @@ void ppack (int fdes, enum ck_msg_type type, CheckMsg *msg)
   ssize_t r;
 
   n = pack (type, &buf, msg);
+  pthread_mutex_lock(&mutex_lock);
   r = write (fdes, buf, n);
+  pthread_mutex_unlock(&mutex_lock);
   if (r == -1)
     eprintf ("Error in call to write:", __FILE__, __LINE__ - 2);
 
