@@ -18,12 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#define _GNU_SOURCE
-#include "config.h"
+#include "../lib/libcompat.h"
 
 #include <sys/types.h>
-#include <sys/wait.h>
-#include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -37,7 +34,6 @@
 #include "check_msg.h"
 #include "check_log.h"
 
-
 enum rinfo {
   CK_R_SIG,
   CK_R_PASS,
@@ -52,40 +48,46 @@ enum tf_type {
   CK_NOFORK_FIXTURE
 };
 
+/* all functions are defined in the same order they are declared.
+   functions that depend on forking are gathered all together.
+   non-static functions are at the end of the file. */
 static void srunner_run_init (SRunner *sr, enum print_output print_mode);
 static void srunner_run_end (SRunner *sr, enum print_output print_mode);
 static void srunner_iterate_suites (SRunner *sr,
 				    enum print_output print_mode);
-static void srunner_run_tcase (SRunner *sr, TCase *tc);
-static int srunner_run_unchecked_setup (SRunner *sr, TCase *tc);
-static void srunner_run_unchecked_teardown (SRunner *sr, TCase *tc);
-static TestResult * tcase_run_checked_setup (SRunner *sr, TCase *tc);
-static void tcase_run_checked_teardown (TCase *tc);
 static void srunner_iterate_tcase_tfuns (SRunner *sr, TCase *tc);
 static void srunner_add_failure (SRunner *sr, TestResult *tf);
-static TestResult *tcase_run_tfun_fork (SRunner *sr, TCase *tc, TF *tf, int i);
+static int srunner_run_unchecked_setup (SRunner *sr, TCase *tc);
+static TestResult * tcase_run_checked_setup (SRunner *sr, TCase *tc);
+static void srunner_run_teardown (List *l);
+static void srunner_run_unchecked_teardown (TCase *tc);
+static void tcase_run_checked_teardown (TCase *tc);
+static void srunner_run_tcase (SRunner *sr, TCase *tc);
 static TestResult *tcase_run_tfun_nofork (SRunner *sr, TCase *tc, TF *tf, int i);
-static TestResult *receive_result_info_fork (const char *tcname,
-                                             const char *tname,
-                                             int iter,
-					     int status, int expected_signal);
 static TestResult *receive_result_info_nofork (const char *tcname,
                                                const char *tname,
                                                int iter);
-static void set_fork_info (TestResult *tr, int status, int expected_signal);
 static void set_nofork_info (TestResult *tr);
+static char *pass_msg (void);
+
+#ifdef _POSIX_VERSION
+static TestResult *tcase_run_tfun_fork (SRunner *sr, TCase *tc, TF *tf, int i);
+static TestResult *receive_result_info_fork (const char *tcname,
+                                             const char *tname,
+                                             int iter,
+					     int status, int expected_signal,
+					     unsigned char allowed_exit_value);
+static void set_fork_info (TestResult *tr, int status, int expected_signal,
+			   unsigned char allowed_exit_value);
 static char *signal_msg (int sig);
 static char *signal_error_msg (int signal_received, int signal_expected);
-static char *pass_msg (void);
 static char *exit_msg (int exitstatus);
 static int waserror (int status, int expected_signal);
 
-#define MSG_LEN 100
-
 static int alarm_received;
 static pid_t group_pid;
 
-static void sig_handler(int sig_nr)
+static void CK_ATTRIBUTE_UNUSED sig_handler(int sig_nr)
 {
   switch (sig_nr) {
    case SIGALRM:
@@ -97,6 +99,9 @@ static void sig_handler(int sig_nr)
     break;
   }
 }
+#endif /* _POSIX_VERSION */
+
+#define MSG_LEN 100
 
 static void srunner_run_init (SRunner *sr, enum print_output print_mode)
 {
@@ -106,7 +111,7 @@ static void srunner_run_init (SRunner *sr, enum print_output print_mode)
   log_srunner_start (sr);
 }
 
-static void srunner_run_end (SRunner *sr, enum print_output print_mode)
+static void srunner_run_end (SRunner *sr, enum print_output CK_ATTRIBUTE_UNUSED print_mode)
 {
   log_srunner_end (sr);
   srunner_end_logging (sr);
@@ -115,7 +120,7 @@ static void srunner_run_end (SRunner *sr, enum print_output print_mode)
 }
 
 static void srunner_iterate_suites (SRunner *sr,
-				    enum print_output print_mode)
+				    enum print_output CK_ATTRIBUTE_UNUSED print_mode)
   
 {
   List *slst;
@@ -140,38 +145,6 @@ static void srunner_iterate_suites (SRunner *sr,
   }
 }
 
-void srunner_run_all (SRunner *sr, enum print_output print_mode)
-{
-  struct sigaction old_action;
-  struct sigaction new_action;
-  
-  if (sr == NULL)
-    return;
-  if (print_mode >= CK_LAST)
-    {
-      eprintf ("Bad print_mode argument to srunner_run_all: %d",
-	      __FILE__, __LINE__, print_mode);
-    }
-  memset(&new_action, 0, sizeof new_action);
-  new_action.sa_handler = sig_handler;
-  sigaction(SIGALRM, &new_action, &old_action);
-  srunner_run_init (sr, print_mode);
-  srunner_iterate_suites (sr, print_mode);
-  srunner_run_end (sr, print_mode);
-  sigaction(SIGALRM, &old_action, NULL);
-}
-
-static void srunner_add_failure (SRunner *sr, TestResult *tr)
-{  
-  list_add_end (sr->resultlst, tr);
-  sr->stats->n_checked++; /* count checks during setup, test, and teardown */
-  if (tr->rtype == CK_FAILURE)
-    sr->stats->n_failed++;
-  else if (tr->rtype == CK_ERROR)
-    sr->stats->n_errors++;
-  
-}
-
 static void srunner_iterate_tcase_tfuns (SRunner *sr, TCase *tc)
 {
   List *tfl;
@@ -186,14 +159,19 @@ static void srunner_iterate_tcase_tfuns (SRunner *sr, TCase *tc)
 
     for (i = tfun->loop_start; i < tfun->loop_end; i++)
     {
+      log_test_start (sr, tc, tfun);
       switch (srunner_fork_status(sr)) {
-       case CK_FORK:
+      case CK_FORK:
+#ifdef _POSIX_VERSION
         tr = tcase_run_tfun_fork (sr, tc, tfun, i);
+#else /* _POSIX_VERSION */
+        eprintf("This version does not support fork", __FILE__, __LINE__);
+#endif /* _POSIX_VERSION */
         break;
-       case CK_NOFORK:
+      case CK_NOFORK:
         tr = tcase_run_tfun_nofork (sr, tc, tfun, i);
         break;
-       default:
+      default:
         eprintf("Bad fork status in SRunner", __FILE__, __LINE__);
       }
       srunner_add_failure (sr, tr);
@@ -202,6 +180,17 @@ static void srunner_iterate_tcase_tfuns (SRunner *sr, TCase *tc)
   }
 }  
 
+static void srunner_add_failure (SRunner *sr, TestResult *tr)
+{  
+  list_add_end (sr->resultlst, tr);
+  sr->stats->n_checked++; /* count checks during setup, test, and teardown */
+  if (tr->rtype == CK_FAILURE)
+    sr->stats->n_failed++;
+  else if (tr->rtype == CK_ERROR)
+    sr->stats->n_errors++;
+  
+}
+
 static int srunner_run_unchecked_setup (SRunner *sr, TCase *tc)
 {
   TestResult *tr;
@@ -270,61 +259,46 @@ static TestResult * tcase_run_checked_setup (SRunner *sr, TCase *tc)
   return tr;
 }
 
-static void tcase_run_checked_teardown (TCase *tc)
+static void srunner_run_teardown (List *l)
 {
-  List *l;
-  Fixture *f;
-
-  l = tc->ch_tflst;
-  
-  send_ctx_info(CK_CTX_TEARDOWN);
-
-  for (list_front(l); !list_at_end(l); list_advance(l)) {
-    f = list_val(l);
-    f->fun();
-  }
-}
-
-static void srunner_run_unchecked_teardown (SRunner *sr, TCase *tc)
-{
-  List *l;
   Fixture *f;
   
-  set_fork_status(CK_NOFORK);
-  l = tc->unch_tflst;
-  
   for (list_front(l); !list_at_end(l); list_advance(l)) {
-    
     f = list_val(l);
     send_ctx_info(CK_CTX_TEARDOWN);
     f->fun ();
   }
-  set_fork_status(srunner_fork_status(sr));
+}
+
+static void srunner_run_unchecked_teardown (TCase *tc)
+{
+  srunner_run_teardown(tc->unch_tflst);
+}
+
+static void tcase_run_checked_teardown (TCase *tc)
+{
+  srunner_run_teardown(tc->ch_tflst);
 }
 
 static void srunner_run_tcase (SRunner *sr, TCase *tc)
 {
   if (srunner_run_unchecked_setup(sr,tc)) {
     srunner_iterate_tcase_tfuns(sr,tc);
-    srunner_run_unchecked_teardown(sr, tc);
+    srunner_run_unchecked_teardown(tc);
   }
 }
 
-static TestResult *receive_result_info_fork (const char *tcname,
-                                             const char *tname,
-                                             int iter,
-					     int status, int expected_signal)
+static TestResult *tcase_run_tfun_nofork (SRunner *sr, TCase *tc, TF *tfun, int i)
 {
   TestResult *tr;
-
-  tr = receive_test_result(waserror(status, expected_signal));
-  if (tr == NULL)
-    eprintf("Failed to receive test result", __FILE__, __LINE__);
-  tr->tcname = tcname;
-  tr->tname = tname;
-  tr->iter = iter;
-  set_fork_info(tr, status, expected_signal);
-
+  
+  tr = tcase_run_checked_setup(sr, tc);
+  if (tr == NULL) {
+    tfun->fn(i);
+    tcase_run_checked_teardown(tc);
+    return receive_result_info_nofork(tc->name, tfun->name, i);
+  }
+  
   return tr;
 }
 
@@ -345,7 +319,75 @@ static TestResult *receive_result_info_nofork (const char *tcname,
   return tr;
 }
 
-static void set_fork_info (TestResult *tr, int status, int signal_expected)
+static void set_nofork_info (TestResult *tr)
+{
+  if (tr->msg == NULL) {
+    tr->rtype = CK_PASS;
+    tr->msg = pass_msg();
+  } else {
+    tr->rtype = CK_FAILURE;
+  }
+}
+
+static char *pass_msg (void)
+{
+  char *msg = emalloc(sizeof("Passed"));
+  strcpy (msg, "Passed");
+  return msg;
+}
+
+#ifdef _POSIX_VERSION
+static TestResult *tcase_run_tfun_fork (SRunner *sr, TCase *tc, TF *tfun, int i)
+{
+  pid_t pid_w;
+  pid_t pid;
+  int status = 0;
+
+  pid = fork();
+  if (pid == -1)
+    eprintf("Error in call to fork:", __FILE__, __LINE__ - 2);
+  if (pid == 0) {
+    setpgid(0, 0);
+    group_pid = getpgrp();
+    tcase_run_checked_setup(sr, tc);
+    tfun->fn(i);
+    tcase_run_checked_teardown(tc);
+    exit(EXIT_SUCCESS);
+  } else {
+    group_pid = pid;
+  }
+
+  alarm_received = 0;
+  alarm(tc->timeout);
+  do {
+    pid_w = waitpid(pid, &status, 0);
+  } while (pid_w == -1);
+  
+  killpg(pid, SIGKILL); /* Kill remaining processes. */
+
+  return receive_result_info_fork(tc->name, tfun->name, i, status, tfun->signal,  tfun->allowed_exit_value);
+}
+
+static TestResult *receive_result_info_fork (const char *tcname,
+                                             const char *tname,
+                                             int iter,
+					     int status, int expected_signal, 
+                                             unsigned char allowed_exit_value)
+{
+  TestResult *tr;
+
+  tr = receive_test_result(waserror(status, expected_signal));
+  if (tr == NULL)
+    eprintf("Failed to receive test result", __FILE__, __LINE__);
+  tr->tcname = tcname;
+  tr->tname = tname;
+  tr->iter = iter;
+  set_fork_info(tr, status, expected_signal, allowed_exit_value);
+
+  return tr;
+}
+
+static void set_fork_info (TestResult *tr, int status, int signal_expected, unsigned char allowed_exit_value)
 {
   int was_sig = WIFSIGNALED(status);
   int was_exit = WIFEXITED(status);
@@ -372,10 +414,10 @@ static void set_fork_info (TestResult *tr, int status, int signal_expected)
       tr->msg = signal_msg(signal_received);
     }
   } else if (signal_expected == 0) {
-    if (was_exit && exit_status == 0) {
+    if (was_exit && exit_status == allowed_exit_value) {
       tr->rtype = CK_PASS;
       tr->msg = pass_msg();
-    } else if (was_exit && exit_status != 0) {
+    } else if (was_exit && exit_status != allowed_exit_value) {
       if (tr->msg == NULL) { /* early exit */
         tr->rtype = CK_ERROR;
         tr->msg = exit_msg(exit_status);
@@ -386,7 +428,7 @@ static void set_fork_info (TestResult *tr, int status, int signal_expected)
   } else { /* a signal was expected and none raised */
     if (was_exit) {
       tr->msg = exit_msg(exit_status);
-      if (exit_status == 0)
+      if (exit_status == allowed_exit_value)
 	tr->rtype = CK_FAILURE; /* normal exit status */
       else
 	tr->rtype = CK_FAILURE; /* early exit */
@@ -394,59 +436,16 @@ static void set_fork_info (TestResult *tr, int status, int signal_expected)
   }
 }
 
-static void set_nofork_info (TestResult *tr)
+static char *signal_msg (int signal)
 {
-  if (tr->msg == NULL) {
-    tr->rtype = CK_PASS;
-    tr->msg = pass_msg();
+  char *msg = emalloc(MSG_LEN); /* free'd by caller */
+  if (alarm_received) {
+    snprintf(msg, MSG_LEN, "Test timeout expired");
   } else {
-    tr->rtype = CK_FAILURE;
+    snprintf(msg, MSG_LEN, "Received signal %d (%s)",
+             signal, strsignal(signal));
   }
-}
-
-static TestResult *tcase_run_tfun_nofork (SRunner *sr, TCase *tc, TF *tfun, int i)
-{
-  TestResult *tr;
-  
-  tr = tcase_run_checked_setup(sr, tc);
-  if (tr == NULL) {
-    tfun->fn(i);
-    tcase_run_checked_teardown(tc);
-    return receive_result_info_nofork(tc->name, tfun->name, i);
-  }
-  
-  return tr;
-}
-
-  
-static TestResult *tcase_run_tfun_fork (SRunner *sr, TCase *tc, TF *tfun, int i)
-{
-  pid_t pid_w;
-  pid_t pid;
-  int status = 0;
-  pid = fork();
-  if (pid == -1)
-    eprintf("Error in call to fork:", __FILE__, __LINE__ - 2);
-  if (pid == 0) {
-    setpgid(0, 0);
-    group_pid = getpgrp();
-    tcase_run_checked_setup(sr, tc);
-    tfun->fn(i);
-    tcase_run_checked_teardown(tc);
-    exit(EXIT_SUCCESS);
-  } else {
-    group_pid = pid;
-  }
-
-  alarm_received = 0;
-  alarm(tc->timeout);
-  do {
-    pid_w = waitpid(pid, &status, 0);
-  } while (pid_w == -1);
-  
-  killpg(pid, SIGKILL); /* Kill remaining processes. */
-
-  return receive_result_info_fork(tc->name, tfun->name, i, status, tfun->signal);
+  return msg;
 }
 
 static char *signal_error_msg (int signal_received, int signal_expected)
@@ -468,18 +467,6 @@ static char *signal_error_msg (int signal_received, int signal_expected)
   return msg;
 }
 
-static char *signal_msg (int signal)
-{
-  char *msg = emalloc(MSG_LEN); /* free'd by caller */
-  if (alarm_received) {
-    snprintf(msg, MSG_LEN, "Test timeout expired");
-  } else {
-    snprintf(msg, MSG_LEN, "Received signal %d (%s)",
-             signal, strsignal(signal));
-  }
-  return msg;
-}
-
 static char *exit_msg (int exitval)
 {
   char *msg = emalloc(MSG_LEN); /* free'd by caller */
@@ -488,12 +475,17 @@ static char *exit_msg (int exitval)
   return msg;
 }
 
-static char *pass_msg (void)
+static int waserror (int status, int signal_expected)
 {
-  char *msg = emalloc(sizeof("Passed"));
-  strcpy (msg, "Passed");
-  return msg;
+  int was_sig = WIFSIGNALED (status);
+  int was_exit = WIFEXITED (status);
+  int exit_status = WEXITSTATUS (status);
+  int signal_received = WTERMSIG (status);
+
+  return ((was_sig && (signal_received != signal_expected)) ||
+          (was_exit && exit_status != 0));
 }
+#endif /* _POSIX_VERSION */
 
 enum fork_status srunner_fork_status (SRunner *sr)
 {
@@ -503,8 +495,14 @@ enum fork_status srunner_fork_status (SRunner *sr)
       return CK_FORK;
     if (strcmp (env,"no") == 0)
       return CK_NOFORK;
-    else
+    else {
+#ifdef _POSIX_VERSION
       return CK_FORK;
+#else /* _POSIX_VERSION */
+      eprintf("This version does not support fork", __FILE__, __LINE__);
+      return CK_NOFORK;
+#endif /* _POSIX_VERSION */
+    }
   } else
     return sr->fstat;
 }
@@ -514,16 +512,49 @@ void srunner_set_fork_status (SRunner *sr, enum fork_status fstat)
   sr->fstat = fstat;
 }
 
+void srunner_run_all (SRunner *sr, enum print_output print_mode)
+{
+#ifdef _POSIX_VERSION
+  struct sigaction old_action;
+  struct sigaction new_action;
+#endif /* _POSIX_VERSION */
+
+  if (sr == NULL)
+    return;
+  if (print_mode >= CK_LAST)
+    {
+      eprintf ("Bad print_mode argument to srunner_run_all: %d",
+	      __FILE__, __LINE__, print_mode);
+    }
+#ifdef _POSIX_VERSION
+  memset(&new_action, 0, sizeof new_action);
+  new_action.sa_handler = sig_handler;
+  sigaction(SIGALRM, &new_action, &old_action);
+#endif /* _POSIX_VERSION */
+  srunner_run_init (sr, print_mode);
+  srunner_iterate_suites (sr, print_mode);
+  srunner_run_end (sr, print_mode);
+#ifdef _POSIX_VERSION
+  sigaction(SIGALRM, &old_action, NULL);
+#endif /* _POSIX_VERSION */
+}
+
 pid_t check_fork (void)
 {
+#ifdef _POSIX_VERSION
   pid_t pid = fork();
   /* Set the process to a process group to be able to kill it easily. */
   setpgid(pid, group_pid);
   return pid;
+#else /* _POSIX_VERSION */
+  eprintf("This version does not support fork", __FILE__, __LINE__);
+  return 0;
+#endif /* _POSIX_VERSION */
 }
 
-void check_waitpid_and_exit (pid_t pid)
+void check_waitpid_and_exit (pid_t pid CK_ATTRIBUTE_UNUSED)
 {
+#ifdef _POSIX_VERSION
   pid_t pid_w;
   int status;
 
@@ -531,19 +562,12 @@ void check_waitpid_and_exit (pid_t pid)
     do {
       pid_w = waitpid(pid, &status, 0);
     } while (pid_w == -1);
-    if (waserror(status, 0))
+    if (waserror(status, 0)) {
       exit(EXIT_FAILURE);
+    }
   }
   exit(EXIT_SUCCESS);
+#else /* _POSIX_VERSION */
+  eprintf("This version does not support fork", __FILE__, __LINE__);
+#endif /* _POSIX_VERSION */
 }  
-
-static int waserror (int status, int signal_expected)
-{
-  int was_sig = WIFSIGNALED (status);
-  int was_exit = WIFEXITED (status);
-  int exit_status = WEXITSTATUS (status);
-  int signal_received = WTERMSIG(status);
-
-  return ((was_sig && (signal_received != signal_expected)) ||
-          (was_exit && exit_status != 0));
-}
