@@ -20,9 +20,14 @@ Suite *make_master_suite(void);
 Suite *make_list_suite(void);
 Suite *make_msg_suite(void);
 Suite *make_log_suite(void);
+Suite *make_log_internal_suite(void);
 Suite *make_limit_suite(void);
 Suite *make_fork_suite(void);
 Suite *make_fixture_suite(void);
 Suite *make_pack_suite(void);
+Suite *make_exit_suite(void);
+
+extern int master_tests_lineno[];
+void init_master_tests_lineno(void);
 
 #endif /* CHECK_CHECK_H */
