@@ -1,4 +1,4 @@
-#define _GNU_SOURCE
+#include "../lib/libcompat.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -11,12 +11,10 @@
 TestResult **tr_fail_array;
 TestResult **tr_all_array;
 
-
 #define MAXSTR 300
 
 typedef struct {
   const char *tcname;
-  int line_nos;
   int failure_type;
   const char *msg;
 } master_test_t;
@@ -26,76 +24,95 @@ static char signal_11_str[SIG_STR_LEN];
 static char signal_11_8_str[SIG_STR_LEN];
 static char signal_8_str[SIG_STR_LEN];
 
+/* FIXME: all these line numbers are kind of hard to maintain */
 static master_test_t master_tests[] = {
-  { "Simple Tests",  18, CK_FAILURE, "Failure expected" },
-  { "Simple Tests",  24, CK_ERROR,   "Early exit with return value 1" },
-  { "Simple Tests",  -1, CK_PASS,    "Passed" },
-  { "Simple Tests",  -1, CK_FAILURE, "This test should fail" },
-  { "Simple Tests",  -1, CK_PASS,    "Passed" },
-  { "Simple Tests",  -1, CK_FAILURE, "This test should fail" },
-  { "Simple Tests",  -1, CK_FAILURE, "Assertion '2 == 3' failed" },
-  { "Simple Tests",  -1, CK_FAILURE, "Assertion '4 == 5' failed" },
-  { "Simple Tests",  -1, CK_FAILURE, "Failure '2 != 3' occured" },
-  { "Simple Tests",  -1, CK_FAILURE, "Failure '4 != 5' occured" },
-  { "Simple Tests",  -1, CK_FAILURE, "3 != 4" },
-  { "Simple Tests",  -1, CK_FAILURE, "5 != 6" },
-  { "Simple Tests",  -1, CK_FAILURE, "7 == 7" },
-  { "Simple Tests",  -1, CK_FAILURE, "Failed" },
-  { "Simple Tests", 114, CK_FAILURE, "Failed" },
-  { "Simple Tests", 120, CK_FAILURE, "Failure expected" },
-  { "Simple Tests", 127, CK_FAILURE, "Failed" },
-  { "Simple Tests", 139, CK_FAILURE, "Assertion 'x == y' failed" },
-  { "Simple Tests", 145, CK_FAILURE, "Assertion '0' failed" },
-  { "Simple Tests", 170, CK_FAILURE, "Assertion 'x==y' failed: x==3, y==4" },
-  { "Simple Tests", 180, CK_FAILURE, "Assertion 'x!=y' failed: x==3, y==3" },
-  { "Simple Tests", 188, CK_FAILURE, "Assertion '\"test1\"==s' failed: \"test1\"==\"test1\", s==\"test2\"" },
-  { "Simple Tests", 198, CK_FAILURE, "Assertion 't!=s' failed: t==\"test2\", s==\"test2\"" },
+  { "Simple Tests", CK_FAILURE, "Failure expected" },
+  { "Simple Tests", CK_ERROR,   "Early exit with return value 1" },
+  { "Simple Tests", CK_PASS,    "Passed" },
+  { "Simple Tests", CK_FAILURE, "This test should fail" },
+  { "Simple Tests", CK_PASS,    "Passed" },
+  { "Simple Tests", CK_FAILURE, "This test should fail" },
+  { "Simple Tests", CK_FAILURE, "Assertion '2 == 3' failed" },
+  { "Simple Tests", CK_FAILURE, "Assertion '4 == 5' failed" },
+  { "Simple Tests", CK_FAILURE, "Failure '2 != 3' occured" },
+  { "Simple Tests", CK_FAILURE, "Failure '4 != 5' occured" },
+  { "Simple Tests", CK_FAILURE, "3 != 4" },
+  { "Simple Tests", CK_FAILURE, "5 != 6" },
+  { "Simple Tests", CK_FAILURE, "7 == 7" },
+  { "Simple Tests", CK_FAILURE, "Failed" },
+  { "Simple Tests", CK_FAILURE, "Failed" },
+  { "Simple Tests", CK_FAILURE, "Failure expected" },
+  { "Simple Tests", CK_FAILURE, "Failed" },
+  { "Simple Tests", CK_FAILURE, "Assertion 'x == y' failed" },
+  { "Simple Tests", CK_FAILURE, "Assertion '0' failed" },
+  { "Simple Tests", CK_FAILURE, "Assertion 'x==y' failed: x==3, y==4" },
+  { "Simple Tests", CK_FAILURE, "Assertion 'x!=y' failed: x==3, y==3" },
+  { "Simple Tests", CK_FAILURE, "Assertion '\"test1\"==s' failed: \"test1\"==\"test1\", s==\"test2\"" },
+  { "Simple Tests", CK_FAILURE, "Assertion 't!=s' failed: t==\"test2\", s==\"test2\"" },
 
-  { "Signal Tests",  -1, CK_ERROR,   signal_11_str },
-  { "Signal Tests",  -1, CK_PASS,    "Passed" },
-  { "Signal Tests", 228, CK_ERROR,   signal_11_8_str },
-  { "Signal Tests",  -1, CK_FAILURE, "Early exit with return value 0" },
-  { "Signal Tests",  -1, CK_FAILURE, "Early exit with return value 1" },
-  { "Signal Tests",  -1, CK_ERROR,   signal_8_str },
-  { "Signal Tests",  -1, CK_ERROR,   signal_8_str },
+  { "Signal Tests", CK_ERROR,   signal_11_str },
+  { "Signal Tests", CK_PASS,    "Passed" },
+  { "Signal Tests", CK_ERROR,   signal_11_8_str },
+  { "Signal Tests", CK_FAILURE, "Early exit with return value 0" },
+  { "Signal Tests", CK_FAILURE, "Early exit with return value 1" },
+  { "Signal Tests", CK_ERROR,   signal_8_str },
+  { "Signal Tests", CK_ERROR,   signal_8_str },
 
 #if TIMEOUT_TESTS_ENABLED
-  { "Environment Timeout Tests", 256, CK_ERROR,  "Test timeout expired" },
-  { "Environment Timeout Tests",  -1, CK_PASS,   "Passed" },
-  { "Environment Timeout Tests",  -1, CK_PASS,   "Passed" },
-  { "Environment Timeout Tests", 275, CK_ERROR,  "Test timeout expired" },
-  { "Timeout Tests", 256, CK_ERROR,  "Test timeout expired" },
-  { "Timeout Tests",  -1, CK_PASS,   "Passed" },
-  { "Timeout Tests", 269, CK_ERROR,  "Test timeout expired" },
-  { "Timeout Tests", 275, CK_ERROR,  "Test timeout expired" },
-  { "User Timeout Tests", 256, CK_ERROR,  "Test timeout expired" },
-  { "User Timeout Tests",  -1, CK_PASS,   "Passed" },
-  { "User Timeout Tests",  -1, CK_PASS,   "Passed" },
-  { "User Timeout Tests", 275, CK_ERROR,  "Test timeout expired" },
+#if HAVE_WORKING_SETENV
+  { "Environment Timeout Tests", CK_ERROR,  "Test timeout expired" },
+  { "Environment Timeout Tests", CK_PASS,   "Passed" },
+  { "Environment Timeout Tests", CK_PASS,   "Passed" },
+  { "Environment Timeout Tests", CK_ERROR,  "Test timeout expired" },
+#endif
+  { "Timeout Tests", CK_ERROR,  "Test timeout expired" },
+  { "Timeout Tests", CK_PASS,   "Passed" },
+  { "Timeout Tests", CK_ERROR,  "Test timeout expired" },
+  { "Timeout Tests", CK_ERROR,  "Test timeout expired" },
+  { "User Timeout Tests", CK_ERROR,  "Test timeout expired" },
+  { "User Timeout Tests", CK_PASS,   "Passed" },
+  { "User Timeout Tests", CK_PASS,   "Passed" },
+  { "User Timeout Tests", CK_ERROR,  "Test timeout expired" },
   /* Timeout tests are run twice , see check_check_sub.c:make_sub_suite() */
-  { "Timeout Tests", 256, CK_ERROR,  "Test timeout expired" },
-  { "Timeout Tests",  -1, CK_PASS,   "Passed" },
-  { "Timeout Tests", 269, CK_ERROR,  "Test timeout expired" },
-  { "Timeout Tests", 275, CK_ERROR,  "Test timeout expired" },
+  { "Timeout Tests", CK_ERROR,  "Test timeout expired" },
+  { "Timeout Tests", CK_PASS,   "Passed" },
+  { "Timeout Tests", CK_ERROR,  "Test timeout expired" },
+  { "Timeout Tests", CK_ERROR,  "Test timeout expired" },
+#if HAVE_WORKING_SETENV
+  { "Environment Timeout Scaling Tests", CK_ERROR,  "Test timeout expired" },
+  { "Environment Timeout Scaling Tests", CK_PASS,   "Passed" },
+  { "Environment Timeout Scaling Tests", CK_PASS,   "Passed" },
+  { "Environment Timeout Scaling Tests", CK_ERROR,  "Test timeout expired" },
+  { "Timeout Scaling Tests", CK_ERROR,  "Test timeout expired" },
+  { "Timeout Scaling Tests", CK_PASS,   "Passed" },
+  { "Timeout Scaling Tests", CK_PASS,   "Passed" },
+  { "Timeout Scaling Tests", CK_ERROR,  "Test timeout expired" },
+  { "User Timeout Scaling Tests", CK_ERROR,  "Test timeout expired" },
+  { "User Timeout Scaling Tests", CK_PASS,   "Passed" },
+  { "User Timeout Scaling Tests", CK_PASS,   "Passed" },
+  { "User Timeout Scaling Tests", CK_ERROR,  "Test timeout expired" },
+#endif
 #endif
 
-  { "Limit Tests",   -1, CK_ERROR,   "Early exit with return value 1" },
-  { "Limit Tests",   -1, CK_FAILURE, "Completed properly" },
-  { "Limit Tests",   -1, CK_FAILURE, "Completed properly" },
+  { "Limit Tests", CK_ERROR,   "Early exit with return value 1" },
+  { "Limit Tests", CK_FAILURE, "Completed properly" },
+  { "Limit Tests", CK_FAILURE, "Completed properly" },
 
-  { "Msg and fork Tests",  -1, CK_PASS,       "Passed" },
-  { "Msg and fork Tests",  -1, CK_FAILURE,    "Expected fail" },
-  { "Msg and fork Tests",  -1, CK_PASS,       "Passed" },
-  { "Msg and fork Tests",  -1, CK_FAILURE,    "Expected fail" },
-  { "Msg and fork Tests",  -1, CK_PASS,       "Passed" },
-  { "Msg and fork Tests",  -1, CK_FAILURE,    "Expected fail" },
+  { "Msg and fork Tests", CK_PASS,       "Passed" },
+  { "Msg and fork Tests", CK_FAILURE,    "Expected fail" },
+  { "Msg and fork Tests", CK_PASS,       "Passed" },
+  { "Msg and fork Tests", CK_FAILURE,    "Expected fail" },
+  { "Msg and fork Tests", CK_PASS,       "Passed" },
+  { "Msg and fork Tests", CK_FAILURE,    "Expected fail" },
 
-  { "Core",          -1, CK_PASS,    "Passed" },
-  { "Core",          -1, CK_FAILURE, "We failed" }
+  { "Core", CK_PASS,    "Passed" },
+  { "Core", CK_FAILURE, "We failed" }
 };
 
 static int nr_of_master_tests = sizeof master_tests /sizeof master_tests[0];
 
+int master_tests_lineno[sizeof master_tests /sizeof master_tests[0]];
+
 START_TEST(test_check_nfailures)
 {
   int i;
@@ -165,7 +182,7 @@ START_TEST(test_check_failure_lnos)
     fail_if(i - passed > sub_nfailed, NULL);
     tr = tr_fail_array[i - passed];
     fail_unless(tr != NULL, NULL);
-    line_no = master_tests[i].line_nos;
+    line_no = master_tests_lineno[i];
     if (line_no > 0 && tr_lno(tr) != line_no) {
       char *emsg = malloc(MAXSTR);
       snprintf(emsg, MAXSTR, "For test %d: Expected lno %d, got %d",
@@ -330,6 +347,7 @@ void setup (void)
   Suite *s = make_sub_suite();
   SRunner *sr = srunner_create(s);
 
+  init_master_tests_lineno();
   init_signal_strings();
 
   srunner_add_suite(sr, make_sub2_suite());
