@@ -1,7 +1,8 @@
+#include "../lib/libcompat.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <unistd.h>
 
 #include "check.h"
 #include "check_pack.h"
@@ -188,6 +189,8 @@ START_TEST(test_pack_loc_limit)
 }
 END_TEST
 
+/* the ppack probably means 'pipe' pack */
+#ifdef _POSIX_VERSION
 START_TEST(test_ppack)
 {
   int filedes[2];
@@ -328,7 +331,6 @@ START_TEST(test_ppack_nofail)
 }
 END_TEST
 
-
 #define BIG_MSG_LEN 1037
 
 START_TEST(test_ppack_big)
@@ -370,6 +372,7 @@ START_TEST(test_ppack_big)
   free (fmsg.msg);
 }
 END_TEST
+#endif /* _POSIX_VERSION */
 
 Suite *make_pack_suite(void)
 {
@@ -387,16 +390,20 @@ Suite *make_pack_suite(void)
   tcase_add_test (tc_core, test_pack_loc);
   tcase_add_test (tc_core, test_pack_ctx);
   tcase_add_test (tc_core, test_pack_len);
+#ifdef _POSIX_VERSION
   tcase_add_test (tc_core, test_ppack);
   tcase_add_test (tc_core, test_ppack_noctx);
   tcase_add_test (tc_core, test_ppack_onlyctx);
   tcase_add_test (tc_core, test_ppack_multictx);
   tcase_add_test (tc_core, test_ppack_nofail);
+#endif /* _POSIX_VERSION */
   suite_add_tcase (s, tc_limit);
   tcase_add_test (tc_limit, test_pack_ctx_limit);
   tcase_add_test (tc_limit, test_pack_fail_limit);
   tcase_add_test (tc_limit, test_pack_loc_limit);
+#ifdef _POSIX_VERSION
   tcase_add_test (tc_limit, test_ppack_big);
+#endif /* _POSIX_VERSION */
 
   return s;
 }
