@@ -1,7 +1,10 @@
+#include "../lib/libcompat.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <check.h>
+#include "config.h"
 
 START_TEST(test_pass)
 {
@@ -47,11 +50,20 @@ static void run_tests (int printmode)
   srunner_free(sr);
 }
 
+static void print_usage(void)
+{
+    printf ("Usage: ex_output (CK_SILENT | CK_MINIMAL | CK_NORMAL | CK_VERBOSE");
+#if ENABLE_SUBUNIT
+    printf (" | CK_SUBUNIT");
+#endif
+    printf (")\n");
+}
+
 int main (int argc, char **argv)
 {
   
   if (argc != 2) {
-    printf ("Usage: ex_output (CK_SILENT | CK_MINIMAL | CK_NORMAL | CK_VERBOSE)\n");
+    print_usage();
     return EXIT_FAILURE;
   }
 
@@ -63,8 +75,12 @@ int main (int argc, char **argv)
     run_tests(CK_NORMAL);
   else if (strcmp (argv[1], "CK_VERBOSE") == 0)
     run_tests(CK_VERBOSE);
+#if ENABLE_SUBUNIT
+  else if (strcmp (argv[1], "CK_SUBUNIT") == 0)
+    run_tests(CK_SUBUNIT);
+#endif
   else {
-    printf ("Usage: ex_output (CK_SILENT | CK_MINIMAL | CK_NORMAL | CK_VERBOSE)\n");
+    print_usage();
     return EXIT_FAILURE;
   }    
     
