@@ -1,3 +1,5 @@
+#include "../lib/libcompat.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -74,7 +76,7 @@ static void run_tests (int printmode)
 }
 
 
-int main (int argc, char **argv)
+int main (int argc CK_ATTRIBUTE_UNUSED, char **argv CK_ATTRIBUTE_UNUSED)
 {
   run_tests(CK_SILENT);		/* not considered in XML output */
 
